#!/usr/bin/perl -w
#
# this script imports a file into the dad system 

# initialization

#use strict;
use vars qw($LOGFUNC $BASE $PROGRAM
            $DB_HOST $DB_USER $DB_PASS $DB_NAME $DB_DRIVER
            $CHECKSUM $QUANTAUDIO $ID3 $VC
            $DAD_USER_ID $SONG_TYPE_ID);

use Encode;
use utf8;
            

$PROGRAM = "dad-import";
my $version = "0.2.5";

my $conf = "/etc/dad.conf";
require $conf;
require $LOGFUNC;

# Getopt stuff
use Getopt::Std;
my %opt;

# database stuff
use DBI;

# programs to get QUANTAUDIO comment
my %getquantaudio =
(
  "mp3" => "id3 -Rl \"%f\" | grep Comment",
  "mp2" => "id3 -Rl \"%f\" | grep Comment",
  "mp1" => "id3 -Rl \"%f\" | grep Comment",
  "ogg" => "vorbiscomment -l \"%f\" | grep -i QUANTAUDIO"
);

# subroutines

sub end_program ($$)
# log exit and exit with return code
{
  my $dbh = shift;
  my $value = shift;

  if ($dbh) { $dbh->disconnect (); }
  logger ("S3", "stopped.");
  exit $value;
}

sub check_arguments ($)
# check arguments and fill in all relevant data in the opt_ref 
{
  my $opt_ref = shift;

  if (!getopts ('hsp:', $opt_ref))
  {
    usage ();
    end_program ("", 1);
  }
  if (!($opt_ref->{"f"} = decode_utf8 ($ARGV[0])))
  {
    # no filename given
    logger ("E3", "no filename given !");
    end_program ("", 1);
  }
  if (! -e $ARGV[0])
  {
    # filename does not exist
    logger ("E3", "file not found !");
    end_program ("", 1);
  }
  
  logger ("I3", "importing file ".$opt_ref->{"f"});
}

sub check_programs ()
# check each of the external programs we depend on
# clear their values if they're not available, logging this
{
  # FIXME: we should find a good way to check if a program is there !
  if ($ID3)
  {
    my $result = system ("$ID3 2> /dev/null > /dev/null"); 
    if ($result != 256)
    {
      logger ("E4", "id3 program $ID3 not available.");
      undef $ID3;
    }
  }
  if ($VC)
  {
    my $result = system ("$VC 2> /dev/null > /dev/null"); 
    if ($result != 256)
    {
      logger ("E4", "vorbiscomment program $VC not available.");
      undef $VC;
    }
  }
}

sub get_pwd ()
# return the current directory
{
 my $pwd = `pwd`;
 chomp $pwd;
 return $pwd;
}

sub connect_database ()
# connect to database, exit if fail
# return the database handler
{
  logger ("D3", "connecting to database");
  my $data_source = "DBI:$DB_DRIVER:$DB_NAME:$DB_HOST";
  # FIXME: not sure about this AutoCommit and relationship with rollback
  my $dbh = DBI->connect ($data_source, $DB_USER, $DB_PASS,
                          { AutoCommit => 1 } );
  if (!$dbh)
  {
    logger ("E3", "could not connect to database");
    end_program ("", 1);
  }
  return $dbh;
}

sub db_get_audiofile_id ($$)
# insert the audiofile into the database and return the id
# fail if the audiofile is already there with the same path !
{
  my $dbh = shift;
  my $filename = shift;

  #print "DEBUG: get_audiofile_id: $filename";
  if ($filename =~ /\//)
  {
    $filename =~ /^.+\/(.+)$/;
    $basefilename = $1;
  }
  else
  {
    $basefilename = $filename;
  }
  my $quotedfilename = decode_utf8 ($dbh->quote ($basefilename));

  my $audiofile_type_id = db_get_audio_filetype_id ($dbh, $filename);

  # check if the path of the file we want to store has an id
  my $path_id = db_get_path_id ($dbh, $filename);
  my ($RMS, $mix_in, $mix_out, $length) = get_audio_parameters ($filename);
  my $user_in = $DAD_USER_ID;
  my $intro = 0;
  my $outro = 0;
  
  my $filesize = (stat ($filename))[7];
  my $checksum = get_checksum ($filename);

  # first check if it's in the database already
  # if it is, exit
  my $query = "SELECT filename FROM audiofile WHERE filename=$quotedfilename AND path_id=$path_id";
#  my $query = "SELECT filename FROM audiofile WHERE filename=$quotedfilename";
  my $sth = $dbh->prepare ($query);
  $sth->execute ();
  if ($sth->rows != 0)
  {
    logger ("E3", "Filename $filename already ".$sth->rows." time(s) in database");
    $sth->finish();
    end_program ($dbh, 1);
  } 
  $query = "INSERT INTO audiofile VALUES('',$audiofile_type_id,$path_id,$quotedfilename,$filesize,'$checksum',$length,$mix_in,$mix_out,$RMS, $intro,$outro,$user_in,'')";
  logger ("I4", "executing query $query");
  $sth = $dbh->prepare ($query);
  $sth->execute ();
  my $rv = $dbh->{'mysql_insertid'}; # the id we just inserted
  return $rv;
}

sub db_get_audio_filetype_id ($$)
# get the audio file type from the filename, query the database for
# the type id
{
  my $dbh = shift;
  my $filename = shift;

  my $extension = (parse_filename ($filename))[3];

  my $query = "SELECT audiofile_type_id FROM audiofiletype WHERE filetype='$extension'";
  my $sth = $dbh->prepare ($query);
  $sth->execute ();
  # FIXME : we should get only one type id !
  my $audiofile_type_id = $sth->fetchrow_array ();
  if (!defined ($audiofile_type_id))
  {
    logger ("E3", "File type $extension not in database");
    end_program ($dbh, 1);
  }
  return $audiofile_type_id;
}

sub db_get_audio_id ($$$)
# put this title in the database and return the audio id
{
  my $dbh = shift;
  my $audiofile_id = shift;
  my $title = $dbh->quote(shift);
  
  my $audio_type_id = $SONG_TYPE_ID;
  my $user_id = $DAD_USER_ID;

  my $query = "INSERT INTO audio VALUES('', $audiofile_id, $audio_type_id, $user_id, $title, 0, 0, 0)";
  logger ("I4", "executing query $query");
  my $sth = $dbh->prepare ($query);
  $sth->execute ();
  my $rv = $dbh->{'mysql_insertid'}; # the id we just inserted
  return $rv;
}

sub db_get_artist_id_list ($@)
# get an artist id for each of the mentioned artists, inserting if necessary
{
  my $dbh = shift;
  my @artist_list = @_;

  my @artist_id_list;
  my $artist;

  foreach $artist (@artist_list)
  {
    $quotedartist = decode_utf8 ($dbh->quote ($artist));
    # query for this artist
    my $query = "SELECT artist_id FROM artist WHERE artistname=$quotedartist";
    my $sth = $dbh->prepare ($query);
    $sth->execute ();
    if ($sth->rows == 0)
    {
      # it's not in there, we need to add it first !
      my $first_letter_id = db_get_first_letter_id ($dbh, $artist);
      #print "DEBUG: first letter id of $artist : $first_letter_id\n";
      my $query = "INSERT INTO artist VALUES ('',$quotedartist,'$first_letter_id')";
      $sth = $dbh->prepare ($query);
      $sth->execute ();
      my $artist_id = $dbh->{'mysql_insertid'}; # the id we just inserted
      logger ("I4", "added $artist to table artist with id $artist_id");
      push @artist_id_list, $artist_id;
    }
    else
    {
      if ($sth->rows != 1)
      {
        # it's more than 1
        logger ("E3", "database problem : $artist ".$sth->rows." times in table");
        end_program ($dbh, 1);
      }
      my $artist_id = $sth->fetchrow_array ();
      push @artist_id_list, $artist_id;
    }
  }
  #print "DEBUG: (@artist_id_list)\n";
  return @artist_id_list;
}

sub db_get_user_id_list ($)
# get all of the user id's in the database
{
  my $dbh = shift;

  my @user_id_list;
  my $user_id;
  
  my $query = "SELECT user_id FROM users";
  my $sth = $dbh->prepare ($query);
  $sth->execute ();
  while ($user_id = $sth->fetchrow ())
  {
    push @user_id_list, $user_id;
  }
  #print "DEBUG: (@user_id_list)\n";
  return @user_id_list;
}

sub db_get_cat_id_list ($)
# get all of the category id's in the database
{
  my $dbh = shift;

  my @cat_id_list;
  my $cat_id;
  
  my $query = "SELECT cat_id FROM category";
  my $sth = $dbh->prepare ($query);
  $sth->execute ();
  while ($cat_id = $sth->fetchrow ())
  {
    push @cat_id_list, $cat_id;
  }
  #print "DEBUG: (@cat_id_list)\n";
  return @cat_id_list;
}

sub db_get_path_id ($$)
# check if the path for this file is in the database yet
# if not, add it
# return the id of the path
{
  my $dbh = shift;
  my $filename = shift;

  my $path = (parse_filename ($filename))[0];
  # strip trailing slash if it's there
  $path =~ s/\/$//;
  my $quoted_path = $dbh->quote ($path);

  my $query = "SELECT path_id FROM paths WHERE path=$quoted_path";
  my $sth = $dbh->prepare ($query);
  $sth->execute ();
  if ($sth->rows == 0)
  {
    # it's not in there, we need to add it first !
    my $query = "INSERT INTO paths VALUES ('',$quoted_path)";
    $sth = $dbh->prepare ($query);
    $sth->execute ();
    my $path_id = $dbh->{'mysql_insertid'}; # the id we just inserted
    logger ("I4", "added $path to table paths with id $path_id");
    return $path_id;
  }
  else
  {
    if ($sth->rows != 1)
    {
      # it's more than 1
      logger ("E3", "database problem : $path ".$sth->rows." times in table");
      end_program ($dbh, 1);
    }
    my $path_id = $sth->fetchrow_array ();
    return $path_id;
  }
}

sub db_get_first_letter_id ($$)
# get id of first letter of artist; create if not there
{
  my $dbh = shift;
  my $artist = shift;
  
  my $first_letter_id;
  
  my $letter = substr ($artist, 0, 1);
  #print "DEBUG: db_get_first_letter_id: letter $letter\n";
  my $query = "SELECT first_letter_id FROM firstletter WHERE letter='$letter'";
  my $sth = $dbh->prepare ($query);
  $sth->execute ();
  #print "DEBUG: db_get_first_letter_id: num of rows ".$sth->rows."\n";
  if ($sth->rows == 0)
  {
    # it's not in there, we need to add it first !
    my $query = "INSERT INTO firstletter VALUES ('','$letter')";
    $sth = $dbh->prepare ($query);
    $sth->execute ();
    $first_letter_id = $dbh->{'mysql_insertid'}; # the id we just inserted
    logger ("I4", "added $letter to table firstletter with id $first_letter_id");
    return $first_letter_id;
  }
  else
  {
    if ($sth->rows != 1)
    {
      # it's more than 1
      logger ("E3", "database problem : $letter ".$sth->rows." times in table");
      end_program ($dbh, 1);
    }
    $first_letter_id = $sth->fetchrow_array ();
  }
  return $first_letter_id;
}

sub db_audio_artist_cross ($$@)
{
  my $dbh = shift;
  my $audio_id = shift;
  my @artist_id_list = @_;

  my $artist_id;

  foreach $artist_id (@artist_id_list)
  {
    my $query = "INSERT INTO audioxartist VALUES ('',$artist_id, $audio_id)";
    $sth = $dbh->prepare ($query);
    $sth->execute ();
  }
}

sub db_insert_unrated ($$)
# insert this audio id for each of the users times each of the categories
{
  my $dbh = shift;
  my $audio_id = shift;

  my $user_id;
  my $cat_id;

  my @user_id_list = db_get_user_id_list ($dbh);
  if (!@user_id_list)
  {
    return;
  }
  my @cat_id_list = db_get_cat_id_list ($dbh);
  if (!@cat_id_list)
  {
    return;
  }

  foreach $user_id (@user_id_list)
  {
    foreach $cat_id (@cat_id_list)
    {
      $query = "INSERT INTO unrated VALUES ($audio_id, $cat_id, $user_id)";
      $sth = $dbh->prepare ($query);
      $sth->execute ();
    }
  }
}
 
sub parse_filename ($)
# parse the filename in path, artists, title, extension
# check if it's in the right path to start from (AUDIO_DIR)
# if not, fail outright, otherwise, strip that bit off
{
  my ($path, $artists, $title, $extension);
  my $inputname = shift;
  
  # FIXME : add more filename checking here
  $inputname =~ /^(.+)\s+-\s+(.+)\.([^\.]+)$/;
  if (!$1 || !$2 || !$3)
  {
    logger ("E3", "could not parse file name into components");
    end_program ("", 1);
  }
  
  my $pathartists = $1;
  $title = $2;
  $extension = $3;

  if ($pathartists =~ /\//)
  {
    # it contains a path separator, so extract
    $pathartists =~ /^(.+)\/(.+)/;
    $path = $1;
    $artists = $2;
  }
  else
  {
    # no path, so it's the current dir
    # FIXME: maybe we should put the current path here ?
    $path = get_pwd ();
    $artists = $pathartists;
  }
  ### now check if $path complies with AUDIO_DIR
  ### $AUDIO_DIR has a trailing slash, normally; so add one to path for
  ### when they're equal
  $path .= "/";
  if ($path !~ /^$AUDIO_DIR(.*)$/)
  {
    logger ("E3", "path $path does not begin with AUDIO_DIR $AUDIO_DIR !");
    end_program ("", 1);
  }
  $path = $1;
  return ($path, $artists, $title, $extension);
}

sub get_audio_parameters ($)
# get audio parameters from file
# either extract them from comment field
# or run quantaudio on it if it's on the system
# FIXME: we should add various methods for getting the parameters in other
# ways
{
  my $filename = shift;

  if ($QUANTAUDIO)
  {
    # execute quantaudio on the file anyway; if it doesn't have a field
    # it will add it
    my $command = "$QUANTAUDIO \"$filename\" > /dev/null";
    system ($command);
  }

  # now get the QUANTAUDIO field/comment
  my $ext = (parse_filename ($filename))[3];
  if ($command = $getquantaudio{$ext})
  {
    $command =~ s/%f/$filename/;
    $result = `$command 2> /dev/null`;
    #print $result;
    if ($result =~ /R:(\d+)-T:(\d+\.\d+):(\d+\.\d+):(\d+\.\d+)/)
    {
      my @rv = ($1, $2, $3, $4);
      return @rv;
    }
    else { logger ("E3", "could not execute $command"); }
  }
  logger ("I3", 
          "no quantaudio comment field found, no timing parameters inserted");
    return (0, 0, 0, 0);
}

sub get_artists ($)
# check if the given artistlist string contains more than one artist
# (separated with &) and return a list of separate artists
{
  my $artists = shift;
  my $artist;

  my @artistlist = split " & ", $artists;

  foreach $artist (@artistlist)
  {
    #print "DEBUG: $artist\n";
  }
  return @artistlist;
}

sub get_checksum ()
# calculate checksum with program defined in $CHECKSUM
{
  my $filename = shift;

  if ($CHECKSUM)
  {
    my $result = `$CHECKSUM \"$filename\"`;
    my $checksum = (split " ", $result)[0];
    return $checksum;
  }
  else
  {
    return "";
  }
}
sub usage ()
# print usage information
{
  print "$PROGRAM $version, (C) thomas\@apestaart.org\n";
  print "\n";
  print <<END;
-h     help
-s     single audio file, imports in a different subtree with begin letters
-p     new patch
END
}

# main program

logger ("S3", "started.");

# check arguments
check_arguments (\%opt);

# check helper programs
check_programs ();

# connect to database
my $dbh = connect_database ();

print "dadimporting ".$opt{"f"}." ...\n";
# get filename components;
my ($path, $artists, $title, $extension) = parse_filename ($opt{"f"});
logger ("D4", "path $path, artists $artists, title $title, extension $extension");
my @artist_list = get_artists ($artists);
#print "DEBUG: main: artist list : (@artist_list)\n";

# put the audiofile in the database
my $audiofile_id = db_get_audiofile_id ($dbh, $opt{"f"});

# put the audio info in the database
my $audio_id = db_get_audio_id ($dbh, $audiofile_id, $title);

my @artist_id_list = db_get_artist_id_list ($dbh, @artist_list);

# put the audio-artist-cross table data
db_audio_artist_cross ($dbh, $audio_id, @artist_id_list);

# insert the unrated data
db_insert_unrated ($dbh, $audio_id);

#print "DEBUG: inserted into database with id $audiofile_id\n";
end_program ($dbh, 0);

