#!/usr/bin/perl -w
#
# this script does all of the stuff needed to go from
# an album in the new directory, possibly with corrections, to it's
# final resting place

# the script takes the dir of the album to import from

# first of all, it checks if it should dadalbumcorrect the album
# second, it moves the dir to the music dir
# third, it imports each of the mp3's or oggs in turn
# fourth, it imports the album in the dad database

use strict;
no bytes;
use utf8;
use Encode;

use vars qw($LOGFUNC $PROGRAM $NEW_DIR $AUDIO_DIR $SCRIPTS_DIR);

$PROGRAM = "dad-new-albumimport";
my $version = "0.2.5";

my $conf = "/etc/dad.conf";
require $conf;
require $LOGFUNC;

# Getopt stuff
use Getopt::Std;
my %opt;

use File::Glob ':glob';
use File::Find;
use File::Basename;

	 
sub check_arguments ($)
# check arguments and fill in all relevant data in the opt_ref
{
  my $opt_ref = shift;
  
  if (!getopts ('h', $opt_ref))
  {
    usage ();
    end_program (1);
  }
}                 

sub parse_filename ($)
# parse the filename in path, artists, title, extension
{
  my ($path, $artists, $title, $extension);
  my $inputname = shift;
  
  # FIXME : add more filename checking here
  $inputname =~ /^(.+)\s+-\s+(.+)\.([^\.]+)$/;
  if (!$1 || !$2 || !$3)
  {
    logger ("E3", "could not parse file name into components");
    end_program (1);
  }
  
  my $pathartists = $1;
  $title = $2;
  $extension = $3;

  if ($pathartists =~ /\//)
  {
    # it contains a path separator, so extract
    $pathartists =~ /^(.+)\/(.+)/;
    $path = $1;
    $artists = $2;
  }
  else
  {
    # no path, so it's the current dir
    # FIXME: maybe we should put the current path here ?
    $path = get_pwd ();
    $artists = $pathartists;
  }
      
  return ($path, $artists, $title, $extension);
}

sub get_artists ($)
# check if the given artistlist string contains more than one artist
# (separated with &) and return a list of separate artists
{
  my $artists = shift;
  my $artist;

  my @artistlist = split " & ", $artists;

  foreach $artist (@artistlist)
  {
    #print "DEBUG: $artist\n";
  }
  return @artistlist;
}

sub get_pwd ()
# return the current directory
{
 my $pwd = `pwd`;
 chomp $pwd;
 return $pwd;
}


sub usage ()
# print usage information
{
  print <<END;

$PROGRAM $version, (C) thomas\@apestaart.org

$PROGRAM [album directory to import]

new albums should be in $NEW_DIR
imported albums end up in $AUDIO_DIR

END
}

sub end_program ($)
# log exit and exit
{
  my $value = shift;
  logger ("S1", "stopped.");
  exit $value;
}

# main program

logger ("S1", "started.");

# check arguments

check_arguments (\%opt);

my $dir = decode_utf8 (shift @ARGV) or die "Please give the album dir to import";
# check for trailing space, delete if it's there
if ($dir =~ /\/$/) { chop $dir; }
if ($dir !~ /^\//) { $dir = get_pwd()."/".$dir; } # get full path in $dir

# get the old m3u
# FIXME: this segfaults my @m3us = glob ("$dir/*.m3u");
my @m3us = ();
# FIXME: we need to pass the utf8_decoded dir to File::Find
# FIXME: File::Find::name returns strings not marked as utf8
find (sub { if (/\.m3u$/) { push @m3us, decode_utf8 ($File::Find::name); } }, encode_utf8 ($dir));
#print (@m3us);
   
if (scalar (@m3us) > 1)
{
  logger ("E1", "more than one original m3u file found.");
  print "More than one original m3u file found !\n";
  end_program (1);
}
if (scalar (@m3us) == 0)
{
  logger ("E1", "no original m3u file found.");
  print "No original m3u file found !\n";
  end_program (1);
}

# we get results from File::Find as non-utf8 strings on rh9, so decode
my $oldm3u = $m3us[0];
chomp $oldm3u;
logger ("D2", "found original m3u file $oldm3u");
  
my $newm3u;
# check if we should run dadalbumcorrect.pl
# we don't use glob ("$dir/*.m3u.new") because of brackets [] issues
# we don't use glob at all anymore because glob is teh suck

my @result = `find \"$dir\" -name *.m3u.new`;

my $number = scalar (@result);
logger ("D2", "found $number files ending in .m3u.new");
if ($number > 1)
{
  logger ("E1", "more than one corrected m3u file found.");
  print "More than one corrected m3u file found !\n";
  end_program (1);
}
if ($number == 0) 
{ 
  logger ("E1", "no corrected m3u file found.");
  print "No corrected m3u file found.\n";
  end_program (1);
}

if ($number == 1)
{
  $newm3u = $result[0]; 
  chomp $newm3u;
  
  logger ("I2", "$SCRIPTS_DIR/dad-albumcorrect.pl on $dir");
  my $command = "$SCRIPTS_DIR/dad-albumcorrect.pl \"$oldm3u\" \"$newm3u\"";
  logger ("D2", "$command");
  if ((system ($command) >> 8) != 0)
  {
    logger ("E2", "$command returned non-zero exit value");
    end_program (1);
  }
  $newm3u =~ /^(.+)\.new$/;
  $newm3u = $1;
}

# now move the dir; 
# dir should be the corrected name so correct it first to $newdir
#print "DEBUG: newm3u $newm3u\n";
my $basedir = dirname (dirname ($newm3u));
my ($name, $fulldir, $ext) = fileparse ($newm3u, '\..*');
my $correcteddir = $basedir."/".$name;
my $newdir = $correcteddir;
$newdir =~ s@^$NEW_DIR@$AUDIO_DIR@g;
#print "DEBUG: corr $correcteddir, new $newdir\n";

logger ("I2", "moving $correcteddir to $newdir");
system ("mv \"$correcteddir\" \"$newdir\"");
if (($? >> 8) != 0) {
  logger ("E2", "Error moving $correcteddir to $newdir");
  end_program (1);
}

# now import all of the songs

my $command;

if (! -d $newdir) {
  logger ("E2", "New directory $newdir not found !");
  end_program (1);
} 
$command = "find \"$newdir\" -type f -name *.mp3 -exec $SCRIPTS_DIR/dad-import.pl {} \\;";
logger ("D2", "Executing $command");
system ($command);
if (($? >> 8) != 0) {
  logger ("E2", "Error running dad-import.pl !");
  end_program (1);
}
$command = "find \"$newdir\" -type f -name *.ogg -exec $SCRIPTS_DIR/dad-import.pl {} \\;";
logger ("D2", "Executing $command");
system ($command);
if (($? >> 8) != 0) {
  logger ("E2", "Error running dad-import.pl !");
  end_program (1);
}

my @parts = split '/', $newm3u;
my $m3ubase = pop @parts;
$newm3u = $newdir."/".$m3ubase;
$command = "$SCRIPTS_DIR/dad-albumimport.pl \"$newm3u\"";
logger ("D4", "$command");
system ($command);
# finish
end_program (0);
