#!/usr/bin/perl -w
#
# this script imports performs a regexp on all of the paths in the 
# database

# initialization

#use strict;
use vars qw($LOGFUNC $BASE $PROGRAM
            $DB_HOST $DB_USER $DB_PASS $DB_NAME $DB_DRIVER
            $CHECKSUM $QUANTAUDIO $ID3i
            $DAD_USER_ID $SONG_TYPE_ID);
            

$PROGRAM = "dadpathchange";
my $version = "0.2.5";

my $conf = "/etc/dad.conf";
require $conf;
require $LOGFUNC;

# Getopt stuff
use Getopt::Std;
my %opt;

# database stuff
use DBI;

# subroutines

sub end_program ($)
# log exit and exit
{
  my $dbh = shift;
  if ($dbh) { $dbh->disconnect (); }
  logger ("S3", "stopped.");
  exit;
}

sub check_arguments ($)
# check arguments and fill in all relevant data in the opt_ref 
{
  my $opt_ref = shift;

  if (!getopts ('h:d', $opt_ref))
  {
    usage ();
    end_program (0);
  }
  if ($opt_ref->{"h"})
  {
    usage ();
    end_program (0);
  }
  if (!($opt_ref->{"f"} = $ARGV[0]))
  {
    # no from given
    logger ("E3", "no to be replaced string given !");
    end_program ("");
  }

  if (!($opt_ref->{"t"} = $ARGV[1]))
  {
    # no to given
    logger ("E3", "no replacement string given !");
    end_program ("");
  }
  
  logger ("I3", "changing ".$opt_ref->{"f"}." to ".$opt_ref->{"t"}." in paths");
}

sub connect_database ()
# connect to database, exit if fail
# return the database handler
{
  logger ("D3", "connecting to database");
  my $data_source = "DBI:$DB_DRIVER:$DB_NAME:$DB_HOST";
  # FIXME: not sure about this AutoCommit and relationship with rollback
  my $dbh = DBI->connect ($data_source, $DB_USER, $DB_PASS,
                          { AutoCommit => 1 } );
  if (!$dbh)
  {
    logger ("E3", "could not connect to database");
    end_program ("");
  }
  return $dbh;
}

sub db_change_path ($$)
# perform a regexp on the path variable
{
  my $dbh = shift;
  my $regexp = shift;

  my $query = "SELECT * FROM paths";
  my $sth = $dbh->prepare ($query);
  $sth->execute ();
  while (($path_id, $path) = $sth->fetchrow ())
  {
    $path =~ s/$opt{"f"}/$opt{"t"}/g; 
    if ($opt{"d"})
    {
      # dry run
      print "id $path_id -> ".$path."\n";
    }
    else
    {
      # do sql
      $path = $dbh->quote ($path);
      $query = "UPDATE paths SET path = $path WHERE path_id=$path_id";
      my $sth2 = $dbh->prepare ($query);
      $sth2->execute ();
    }
  }
}

sub usage ()
# print usage information
{
  print "$PROGRAM $version, (C) thomas\@apestaart.org\n";
  print "\n";
  print <<END;
apply the given regexp to each of the paths

-h	help
-d	dry run (don't apply the regexp, only show effects)
END
}

# main program

logger ("S3", "started.");

# check arguments
check_arguments (\%opt);

# connect to database
my $dbh = connect_database ();

db_change_path ($dbh, "/test/");
end_program ($dbh);
