#!/usr/bin/perl -w
#
# this script gets a playlist based on certain parameters
# let's start out simple, shall we ?

# initialization

#use strict;
use vars qw($LOGFUNC $BASE $PROGRAM $AUDIO_DIR
            $DB_HOST $DB_USER $DB_PASS $DB_NAME $DB_DRIVER
            $DAD_USER_ID $SONG_TYPE_ID);
            

$PROGRAM = "dad-playlist";
my $version = "0.2.5";

my $conf = "/etc/dad.conf";
require $conf;
require $LOGFUNC;

# Getopt stuff
use Getopt::Std;
my %opt;

# database stuff
use DBI;

# utf8
use utf8;

# randomize seed
srand;

# subroutines

sub end_program ($)
# log exit and exit
{
  my $dbh = shift;
  if ($dbh) { $dbh->disconnect (); }
  logger ("S3", "stopped.");
  exit;
}

sub check_arguments ($)
# check arguments and fill in all relevant data in the opt_ref 
# s for size
# h for help
# c for category
# r for rating range
# u for user
{
  my $opt_ref = shift;

  if (!getopts ('hs:c:r:u:', $opt_ref))
  {
    print "Error parsing command-line options.\n";
    usage ();
    end_program (0);
  }
  # help
  if ($opt_ref->{"h"}) { usage (); end_program (0); }
  # check size
  if (!$opt_ref->{"s"}) { $opt_ref->{"s"} = 10; } #FIXME : get default
  return;
  #FIXME : do something with the parameters
  if (!($opt_ref->{"f"} = $ARGV[0]))
  {
    # no filename given
    logger ("E3", "no filename given !");
    usage ();
    end_program ("");
  }
  if (! -e $ARGV[0])
  {
    # filename does not exist
    logger ("E3", "file not found !");
    end_program ("");
  }
  
  logger ("I3", "importing file ".$opt_ref->{"f"});
}

### general database functions

sub connect_database ()
# connect to database, exit if fail
# return the database handler
{
  logger ("D3", "connecting to database");
  my $data_source = "DBI:$DB_DRIVER:$DB_NAME:$DB_HOST";
  # FIXME: not sure about this AutoCommit and relationship with rollback
  my $dbh = DBI->connect ($data_source, $DB_USER, $DB_PASS,
                          { RaiseError=> 1, AutoCommit => 1 } );
  if (!$dbh)
  {
    logger ("E3", "could not connect to database");
    end_program ("");
  }
  return $dbh;
}

sub db_row_count ($$$)
# count the number of rows in the table with the given where clause
# 2) table to count rows in
# 3) where clause to limit search
{
  my $dbh = shift;
  my $table = shift;
  my $where = shift;

  $query = "SELECT * FROM $table WHERE $where";
  my $sth = $dbh->prepare ($query);
  $sth->execute ();
  my $row_count = $sth->rows;
}

sub get_table_value_from_id ($$$$$)
# get the value in this table from column $value_column for the given id 
# don't prostitute this function by doing something else than id querying
# 2) the table to get the value in
# 3) the name of the column whose value to get
# 4) the name of the id column to check the id against
# 5) the id
{
  my $dbh = shift;
  my $table = shift;
  my $value_column = shift;
  my $id_column = shift;
  my $id = shift;
  
  my $query = "SELECT $value_column FROM $table WHERE $id_column=$id"; 
  #print "DEBUG: query: $query\n";
  my $sth = $dbh->prepare ($query);
  $sth->execute ();
  my $value = $sth->fetchrow_array ();
  return $value;
}

sub get_table_value_from_where ($$$$)
# get the value in this table from column $value_column for the given $where 
# 2) the table to get the value in
# 3) the name of the column whose value to get
# 4) the where clause to use
{
  my $dbh = shift;
  my $table = shift;
  my $value_column = shift;
  my $where = shift;
  
  my $query = "SELECT $value_column FROM $table WHERE $where"; 
  #print "DEBUG: query: $query\n";
  my $sth = $dbh->prepare ($query);
  $sth->execute ();
  my $value = $sth->fetchrow_array ();
  return $value;
}

sub db_get_all_row_column ($$$$)
# get a column value for all rows in a table
# returns array with all the column values
{
  my $dbh = shift;
  my $table = shift;
  my $column = shift;
  my $where = shift;

  my $query = "SELECT $column FROM $table ";
  if ($where) { $query .= "WHERE $where "; }
  #print "DEBUG: query: $query\n";
  my $sth = $dbh->prepare ($query);
  $sth->execute ();
  my @result;
  while (my $value = $sth->fetchrow_array ()) { push @result, $value; }
  return @result;
}


### specific database functions

sub db_get_audiofile_id_from_audio_id ($$)
# look up the audio filename in the database
# return the audiofile_id
{
  my $dbh = shift;
  my $audio_id = shift;

  my $query = "SELECT audiofile_id FROM audio WHERE audio_id=$audio_id";
  my $sth = $dbh->prepare ($query);
  $sth->execute ();
  if ($sth->rows == 0)
  {
    logger ("E3", "audio with id $audio_id doesn't have an audiofile");
    return 0;
  }
  my $audiofile_id = $sth->fetchrow_array ();
  return $audiofile_id;
}

sub db_get_audio_filetype_id ($$)
# get the audio file type from the filename, query the database for
# the type id
{
  my $dbh = shift;
  my $filename = shift;

  my $extension = (parse_filename ($filename))[3];

  my $query = "SELECT audiofile_type_id FROM audiofiletype WHERE filetype='$extension'";
  my $sth = $dbh->prepare ($query);
  $sth->execute ();
  # FIXME : we should get only one type id !
  my $audiofile_type_id = $sth->fetchrow_array ();
  if (!defined ($audiofile_type_id))
  {
    logger ("E3", "File type $extension not in database");
    end_program ($dbh);
  }
  return $audiofile_type_id;
}

sub db_get_audio_id ($$)
# get the audio id from the audiofile id supplied
{
  my $dbh = shift;
  my $audiofile_id = shift;

  my $query = "SELECT audio_id FROM audio WHERE audiofile_id=$audiofile_id";
  my $sth = $dbh->prepare ($query);
  $sth->execute ();
  if ($sth->rows != 1)
  {
    logger ("E3", "database : ".$sth->rows." audio items found for audiofile_id $audiofile_id");
    end_program ($dbh);
  }
  my $audio_id = $sth->fetchrow_array;
  return $audio_id;
}

sub db_get_cat_id_list ($)
# get all of the category id's in the database
{
  my $dbh = shift;

  my @cat_id_list;
  my $cat_id;
  
  my $query = "SELECT cat_id FROM category";
  my $sth = $dbh->prepare ($query);
  $sth->execute ();
  while ($cat_id = $sth->fetchrow ())
  {
    push @cat_id_list, $cat_id;
  }
  #print "DEBUG: (@cat_id_list)\n";
  return @cat_id_list;
}

sub db_get_audio_artist_id_list ($$)
# get a list of all the artist id's connected to the audio
{
  my $dbh = shift;
  my $audio_id = shift;
  my @artist_id_list;

  my $artist_id;

  my $query = "SELECT artist_id FROM audioxartist WHERE audio_id=$audio_id";
  my $sth = $dbh->prepare ($query);
  $sth->execute ();
  while ($artist_id = $sth->fetchrow_array ())
  {
    push @artist_id_list, $artist_id;
  }
  return @artist_id_list;
}

sub db_get_audiofile ($$)
# get the audio file with path for this id
{
  my $dbh = shift;
  my $audiofile_id = shift;
  
  my $query = "SELECT filename, path_id FROM audiofile WHERE audiofile_id=$audiofile_id";
  my $sth = $dbh->prepare ($query);
  $sth->execute ();
  my ($filename, $path_id) = $sth->fetchrow_array ();
  my $filepath = get_table_value_from_id ($dbh, "paths", "path", "path_id", $path_id);
  return "$filepath/$filename";
}

### general helper functions

# fisher_yates_shuffle( \@array ) : generate a random permutation
# of @array in place
# lifted from perl cookbook
sub fisher_yates_shuffle {
    my $array = shift;
    my $i;
    for ($i = @$array; --$i; ) {
        my $j = int rand ($i+1);
        next if $i == $j;
        @$array[$i,$j] = @$array[$j,$i];
    }
}

### FIXME: this function is not used anymore, remove it
sub db_get_random_row_column ($$$$)
# get a column value from a random row in a table
{
  my $dbh = shift;
  my $table = shift;
  my $column = shift;
  my $where = shift;

  my $query = "SELECT $column FROM $table ";
  if ($where) { $query .= "WHERE $where "; }
  $query .= "ORDER BY RAND() LIMIT 1";
  my $sth = $dbh->prepare ($query);
  $sth->execute ();
  my $value = $sth->fetchrow_array ();
  return $value;
}

sub db_get_category_id ($$)
# return a cat id based on the category name
# return 0 if there's any error
{
  my $dbh = shift;
  my $category = shift;

  if (!$category) { return 0; }

  my $id = get_table_value_from_where  ($dbh, "category", "cat_id", 
                                        "description='$category'");
  return $id;
}

sub generate_playlist ($$$$)
# the main work horse
# generate a playlist from the database
# take artist separation into account
# each artist only once in a playlist
# FIXME: do better selection of categories somehow
{
  my $dbh = shift;
  my $audio_num = shift;
  my $category = shift;
  my $score = shift;

  my %artist_id_list;
  my %audio_id_hash;
  my $i = 0;

  # get a category id based on category passed if there is one
  my $cat_id = db_get_category_id ($dbh, $category);
  my @song_ids;

  # FIXME: the regexp should be better than this so that it works in all cases
  if (!$score) { $score = "0.5-1.0"; }
  $score =~ /(\d*.\d*)-(\d*.\d*)/;
  my $lower = $1 || "0.5";
  my $upper = $2 || "1.0";
  my $scorewhere = "rate >= $lower AND rate <= $upper";

  #print "DEBUG: scores from $lower to $upper\n";
  #print "DEBUG: scorewhere $scorewhere\n";
  # get a list of all audio ids and shuffle it
  if ($cat_id)
  {
    @song_ids = db_get_all_row_column ($dbh, "audio,score", "score.audio_id", 
                   "audio.audio_id=score.audio_id AND ".
                   "audio_type_id=$SONG_TYPE_ID AND ".
                   "cat_id=$cat_id AND $scorewhere");
  }
  else
  {
    @song_ids = db_get_all_row_column ($dbh, "audio", "audio_id", 
                                          "audio_type_id=$SONG_TYPE_ID");
  }
  if (!@song_ids)
  {
    print STDERR "No songs in the database !\n";
    die;
  }
  fisher_yates_shuffle (\@song_ids);
  
  while ($i < $audio_num)
  {
    # if we're out of songs, error out !
    if (!@song_ids)
    {
      print STDERR "No songs left that match the criteria !\n";
      $i = $audio_num;
      next;
    }
    # get a random candidate from the full list of songs
    $song_id = shift @song_ids;
    #print "DEBUG: got a song with id $song_id\n";
    if ($audio_id_hash{$song_id}) { next; } # try again, we had it already

    # check if the candidate is from an already picked artist
    my $artist_id;
    my @artist_ids = db_get_audio_artist_id_list ($dbh, $song_id);
    my $dumpsong = 0; # keep this song unless otherwise
    foreach $artist_id (@artist_ids)
    {
      if ($artist_id_list{$artist_id})
      {
        #print "DEBUG: we already have artist id $artist_id in the list, dumping song...\n";
        $dump_song = 1;
        next;
      }
      $artist_id_list{$artist_id} = 1; # don't want this artist anymore
    }
    # check if it matches the range of rating scores
    if ($dump_song) { $dump_song = 0; next };
    
    # this one's ok, add it
    $audio_id_hash{$song_id} = 1;
    ++$i;
  }

  # we have a hash of audio id's; now get the corresponding files
  my @audio_files;
  foreach $song_id (keys %audio_id_hash)
  {
    #print "DEBUG: looking up file for song $song_id\n";
    my $audiofile_id = db_get_audiofile_id_from_audio_id ($dbh, $song_id);
    if (! $audiofile_id) { next; } #FIXME
    #print "DEBUG: adding audiofile id $audiofile_id\n";
    my $audiofile = $AUDIO_DIR.db_get_audiofile ($dbh, $audiofile_id);
    # if the file exists, we push the utf8-decoded path
    if (-e $audiofile)
    { push @audio_files, decode_utf8 ($audiofile); }
    else
    { print STDERR "ERROR: file $audiofile not on disk !\n"; }
  }
  return @audio_files;
}

sub generate_category_list ($)
# generate a list of categories in database
{
  my $dbh = shift;

  my %artist_id_list;
  my %audio_id_hash;
  my $i = 0;

  # get a list of all audio ids and shuffle it
  my @categories = db_get_all_row_column ($dbh, "category", "description", "");
  foreach $cat (@categories) { print "$cat\n"; }
}
 
sub usage ()
# print usage information
{
  print "$PROGRAM $version, (C) thomas\@apestaart.org\n";
  print "\n";
  print <<END;
-h     help
-s     size of playlist
END
}

# main program

logger ("S3", "started.");

# check arguments
check_arguments (\%opt);

# check helper programs
#check_programs ();

# connect to database
my $dbh = connect_database ();

# check if user asked for a list of categories
if (defined ($opt{"c"}) && $opt{"c"} eq "list")
{
  generate_category_list ($dbh);
  end_program ($dbh);
}

@playlist = generate_playlist ($dbh, $opt{"s"}, $opt{"c"}, $opt{"r"});

foreach $line (@playlist)
{
  print "$line\n";
}
end_program ($dbh);

