<?php
/* 
 * vi:si:et:sw=4:sts=4:ts=4
 *
 * DAD - Digital Audio Database
 *
 * DAD.php: basic functions, error handling, defines, ...
 *
 *  Copyright (C) 2004 Thomas Vander Stichele
 *  Copyright (C) 2004 Kristof De Jaegher
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *
 */

require_once "PEAR.php";
/* FIXME: make PEAR errors fatal until we figure out to handle them properly */
PEAR::setErrorHandling(PEAR_ERROR_DIE);

// {{{ constants
// {{{ error codes
/*
 * If you add an error code here, make sure you also add a textual
 * version of it in DAD::errorMessage().
 */

define("DAD_OK",                         1);
define("DAD_ERROR",                     -1);
define("DAD_ERROR_TABLE_NOT_FOUND",     -2);
define("DAD_ERROR_ID_NOT_FOUND",        -3);
// }}}
// }}}

// {{{ class DAD
/**
 * The main "DAD" class is a class that wraps the DAD database as a whole.
 * It contains methods to get and cache requests for various tables.
 * It also contains some some utility functions
 * common to all parts of DAD.
 */

class DAD
{

    // {{{ DAD()
    /**
     * Constructor.
     *
     * create a new DAD object.
     */
    function DAD()
    {
        /* FIXME: find a way to do this better */
        global $daddb;
        $this->daddb = $daddb;
    }

    // {{{ errorMessage()
    /**
     * Return a textual error message for a DAD error code
     *
     * @param integer $value error code
     *
     * @return string error message, or false if the error code was
     * not recognized
     */
    function errorMessage($value)
    {
        static $errorMessages;
        if (!isset($errorMessages)) {
            $errorMessages = array(
                DAD_ERROR                    => 'unknown error',
                DAD_ERROR_TABLE_NOT_FOUND    => 'table not found',
                DAD_ERROR_ID_NOT_FOUND       => 'id not found'
            );
        }

        return isset($errorMessages[$value]) ? $errorMessages[$value] : $errorMessages[DAD_ERROR];
    }
    // }}}

    // {{{ raiseError()
    /**
     * Raises a PEAR Error.
     *
     * @param integer $value error code
     *
     * @return string error message, or false if the error code was
     * not recognized
     */
    function raiseError($code = DAD_ERROR, $debuginfo = null)
    {
        $string = "DAD Error: " . DAD::errorMessage($code);
        if (isset ($debuginfo))
            $string .= " ($debuginfo)";
        PEAR::raiseError($string, $code);
    }
    // }}}

    // {{{ getTableIds()
    /**
     * Return all the id's from the given table from the given id column.
     *
     * @param string $table table
     * @param string $column id column
     *
     * @return array all id's in that column in the table.
     */
     function getTableIds($table, $column)
     {
         $this->daddb->begin();
         /* FIXME: is there a way to optimize this so we just get the array
          * directly from the query ? */
         $query = "SELECT DISTINCT $column FROM $table";
         $ret = array_keys($this->daddb->all_rows_keyed($query, $column));
         $this->daddb->commit();
         return $ret;
     }
    // }}}

    // {{{ getUserIds()
    /**
     * Return all the user id's in DAD.
     *
     * @return reference reference to array of all user id's in DAD.
     */
     function &getUserIds()
     {
         if (!isset($this->user_ids)) {
             $this->user_ids = $this->getTableIds("users", "user_id");
         }
         return $this->user_ids;
     }
    // }}}

    // {{{ getCatIds()
    /**
     * Return all the category id's in DAD.
     *
     * @return reference reference to array of all category id's in DAD.
     */
     function &getCatIds()
     {
         if (!isset($this->cat_ids)) {
             $this->cat_ids = $this->getTableIds("category", "cat_id");
         }
         return $this->cat_ids;
     }
    // }}}
}
/// }}}
?>
