<?php
/* 
 * vi:si:et:sw=4:sts=4:ts=4
 *
 * DAD - Digital Audio Database
 *
 * DAD_album.php: class for objects from album table
 *
 *  Copyright (C) 2004 Thomas Vander Stichele
 *  Copyright (C) 2004 Kristof De Jaegher
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *
 */

require_once "DAD/object.php";

class DAD_album extends DAD_object
{
    /**
     * DAD_album constructor.
     *
     * @access public
     */
    function DAD_album($id)
    {
        $this->DAD_object("album", "album_id", $id);
    }

    /**
     * Get the displayable title of the album.
     *
     * @return title of the album.
     */
    function getTitle()
    {
        return stripslashes ($this->row->title);
    }

    /**
     * Get a hyperlink HTML snippet linking to the album.
     * FIXME: this currently links to a viewartist browse page without artist.
     * FIXME: rename to getLink
     *
     * @return hyperlink fragment that links to the album.
     */
    function getLink()
    {
        $ret  = "<A href=\"browse.php?dad=viewartist&album_id=";
        $ret .= $this->id . "\">";
        $ret .= $this->getTitle();
        $ret .= "</A>";
        return $ret;
    }

    /**
     * Get a hyperlink HTML snippet for a listen/stream link to the album.
     *
     * @param int user_id user_id for which the playlist is called.
     * @param int type type of stream/play (1 for stream, 2 for local play)
     * @param string alt text describing the type of stream
     *
     * @return hyperlink fragment that links to stream of the album.
     */
     /* FIXME: alt text should really be fixed and in a table */
    function getPlayLink($user_id, $type, $alt)
    {
        if ($type < 0 || $type > 2) {
            DAD::raiseError(DAD_ERROR, "getPlayLink: type $type invalid");
        }
        $ret  = "<A href=\"playlist.m3u?dad=album&dadid=$user_id&album_id=";
        $ret .= $this->id . "&type=$type\">";
        $ret .= "<img src=\"images/sound$type.gif\" border=0 ";
        $ret .= "alt=\"$alt\" title=\"$alt\"></A>";
        return $ret;
    }
}
?>
