<?php
/* 
 * vi:si:et:sw=4:sts=4:ts=4
 *
 * DAD - Digital Audio Database
 *
 * DAD_artist.php: class for objects from artist table
 *
 *  Copyright (C) 2004 Thomas Vander Stichele
 *  Copyright (C) 2004 Kristof De Jaegher
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *
 */

require_once "DAD/object.php";

class DAD_artist extends DAD_object
{
    /**
     * DAD_artist constructor.
     *
     * @access public
     */
    function DAD_artist($id)
    {
        if ($id <= 0)
            DAD::raiseError(DAD_ERROR, "DAD_artist: invalid id '$id'");
        $this->DAD_object("artist", "artist_id", $id);
    }

    /**
     * Get the displayable name of the artist.
     *
     * @return name of the artist.
     */
    function getName()
    {
        return stripslashes ($this->row->artistname);
    }

    /**
     * Get a hyperlink HTML snippet linking to the artist.
     *
     * @return hyperlink fragment that links to the artist.
     */
    function getLink()
    {
        $ret  = "<A href=\"browse.php?dad=viewartist&artist_id=";
        $ret .= $this->id . "\">";
        $ret .= $this->getName();
        $ret .= "</A>";
        return $ret;
    }

    /**
     * Get a list of album id's linked to this artist.
     *
     * @return list of album id's, ordered by album title.
     */
    function getAlbumIds()
    {
        $ret = array();
        
        $query  = "SELECT DISTINCT albumxartist.album_id ";
        $query .= "FROM album,albumxartist ";
        $query .= "WHERE album.album_id=albumxartist.album_id ";
        $query .= "AND artist_id=" . $this->id . " ";
        $query .= "ORDER BY title ASC";
        $rows = $this->db->all_rows($query);
        foreach ($rows as $row) {
            array_push($ret, $row->album_id);
        }
        return $ret;
    }
    /**
     * Get a list of audio id's linked to this artist, ordered by audio title.
     *
     * @return list of audio id's, ordered by audio title.
     */
    function getAudioIds()
    {
        $ret = array();
        
        $query  = "SELECT audioxartist.audio_id ";
        $query .= "FROM audioxartist ";
        $query .= "LEFT JOIN audio ON audioxartist.audio_id=audio.audio_id ";
        $query .= "WHERE artist_id=" . $this->id . " ";
        $query .= "ORDER BY audio.title";

        debug ("DAD_artist->get_AudioIds(): query $query");
        $rows = $this->db->all_rows($query);
        foreach ($rows as $row) {
            array_push($ret, $row->audio_id);
        }
        return $ret;
    }
}
?>
