<?php
/* 
 * vi:si:et:sw=4:sts=4:ts=4
 *
 * DAD - Digital Audio Database
 *
 * DAD_Audio.php: class for objects from audio table
 *
 *  Copyright (C) 2004 Thomas Vander Stichele
 *  Copyright (C) 2004 Kristof De Jaegher
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *
 */

require_once "DAD/object.php";
require_once "DAD/album.php";
require_once "DAD/artist.php";

class DAD_audio extends DAD_object
{
    /**
     * DAD_audio constructor.
     *
     * @access public
     */
    function DAD_audio($id)
    {
        $this->DAD_object("audio", "audio_id", $id);
    }

    /**
     * Get the displayable title of the audio.
     *
     * @return title of the audio.
     */
    function getTitle()
    {
        return stripslashes ($this->row->title);
    }

    /**
     * Get a hyperlink HTML snippet linking to the audio.
     *
     * @return hyperlink fragment that links to the audio.
     */
    function getLink()
    {
        $ret  = "<A href=\"browse.php?dad=songdetail&audio_id=";
        $ret .= $this->id . "\">";
        $ret .= $this->getTitle();
        $ret .= "</A>";
        return $ret;
    }

    /**
     * Get a string of hyperlink HTML snippets for adding the given audio
     * to temp/last/saved playlists for the given user id.
     *
     * @param int user_id user_id for which to add to playlists.
     * @param int pane_id pane_id where we're currently browsing.
     *
     * @return hyperlink fragment to add given audio to temp/last/saved
     * playlists.
     */
    function getAddPlaylistLinks($user_id, $pane_id)
    {
        global $daddb;

        /* FIXME: rewrite this when we merge playlist types */

        $result = "";
        /* if it's not in the temp playlist, add a link */
        $query  = "SELECT audio_id ";
        $query .= "FROM temp_playlist ";
        $query .= "WHERE audio_id=" . $this->id . " ";
        $query .= "AND user_id=$user_id";
        debug ("DAD_audio.getAddPlaylistLinks: query $query");
        $daddb->query($query);
        $row = $daddb->row();
        /* if no result is found, show a link to add to temp playlist */
        if (!$row) {
            $result .= '<A href="browse.php?dad=addtolist';
            $result .= "&dadid=$user_id";
            $result .= '&audio_id=' . $this->id;
            $result .= '&type=temp';
            $result .= "&pane_id=$pane_id";
            $result .= "\">T</A> ";
        }

        /* if it's not in the last playlist, add a link */
        $query  = "SELECT audio_id ";
        $query .= "FROM last_playlist ";
        $query .= "WHERE audio_id=" . $this->id . " ";
        $query .= "AND user_id=$user_id";
        debug ("DAD_audio.getAddPlaylistLinks: query $query");
        $daddb->query($query);
        $row = $daddb->row();
        /* if no result is found, show a link to add to last playlist */
        if (!$row) {
            $result .= '<A href="browse.php?dad=addtolist';
            $result .= "&dadid=$user_id";
            $result .= '&audio_id=' . $this->id;
            $result .= '&type=last';
            $result .= "&pane_id=$pane_id";
            $result .= "\">L</A> ";
        }
        /* if it's not in a saved playlist, add a link */
        /* first get a list of all playlist id's */
        $query  = "SELECT DISTINCT playlist_id ";
        $query .= "FROM save_playlist ";
        $query .= "WHERE audio_id=" . $this->id . " ";
        $query .= "AND user_id=$user_id";
        debug ("DAD_audio.getAddPlaylistLinks: query $query");
        $rows = $daddb->all_rows($query);

        /* for each of the playlist_id's, check if audio is in it and
         * show an add link if not */
        for ($i = 0; $i < count ($rows); $i++) {
            $playlist_id = $rows[$i]->playlist_id;
            $query  = "SELECT audio_id FROM save_playlist ";
            $query .= "WHERE audio_id=" . $this->id . " ";
            $query .= "AND user_id=$user_id";
            debug ("DAD_audio.getAddPlaylistLinks: query $query");
            $daddb->query($query);
            $row = $daddb->row();
            /* if no result is found, show a link to add to last playlist */
            if (!$row) {
                $result .= '<A href="browse.php?dad=addtolist';
                $result .= "&dadid=$user_id";
                $result .= '&audio_id=' . $this->id;
                $result .= '&type=save';
                $result .= "&playlistid=$playlist_id";
                $result .= "&pane_id=$pane_id";
                /* FIXME: show the name, not the id */
                $result .= "\">Pl$playlist_id</A> ";
            }
        }
        return $result;
    }

    /**
     * Get a hyperlink HTML snippet for a listen/stream link to the audio.
     *
     * @param int user_id user_id for which the playlist is called.
     * @param int type type of stream/play (1 for stream, 2 for local play)
     * @param string alt text describing the type of stream
     *
     * @return hyperlink fragment that links to stream of the audio.
     */
     /* FIXME: alt text should really be fixed and in a table */
    function getPlayLink($user_id, $type, $alt)
    {
        if ($type < 0 || $type > 2) {
            DAD::raiseError(DAD_ERROR, "getPlayLink: type $type invalid");
        }
        /* FIXME: change "song" to "audio" */
        $ret  = "<A href=\"playlist.m3u?dad=song&dadid=$user_id&audio_id=";
        $ret .= $this->id . "&type=$type\">";
        $ret .= "<img src=\"images/sound$type.gif\" border=0 ";
        $ret .= "alt=\"$alt\" title=\"$alt\"></A>";
        return $ret;
    }
    /**
     * Get a list of artist id's linked to this audio, ordered by artist name.
     *
     * @return list of artist id's, ordered by artist name.
     */
    function getArtistIds()
    {
        /* return them if we already looked them up */
        if (isset($this->artist_ids))
            return $this->artist_ids;
            
        $this->artist_ids = array();
        
        $query  = "SELECT audioxartist.artist_id ";
        $query .= "FROM audioxartist ";
        $query .= "LEFT JOIN artist ";
        $query .= "ON audioxartist.artist_id=artist.artist_id ";
        $query .= "WHERE audio_id=" . $this->id . " ";
        $query .= "ORDER BY artist.artistname";

        debug("DAD_artist->get_ArtistIds(): query $query");
        $rows = $this->db->all_rows($query);
        if (!$rows) {
            print("ERROR: DAD_artist->get_ArtistIds(): query $query failed");
            exit (1);
        }
        foreach ($rows as $row) {
            array_push($this->artist_ids, $row->artist_id);
        }
        return $this->artist_ids;
    }
    /**
     * Get the type from the audiotype table for this audio.
     *
     * @return type of audio.
     */
    /* FIXME:rename audiotype to type ? */
    function getType()
    {
        global $daddb;
        $query  = "SELECT audiotype FROM audiotype ";
        $query .= "WHERE audio_type_id=" . $this->row->audio_type_id;
        $rows = $daddb->all_rows($query);
        $results = count($rows);
        if ($results != 1)
            DAD::raiseError(DAD_ERROR,
                "DAD_audio: $results results for query $query");
        return stripslashes($rows[0]->audiotype);
    }

    /**
     * Get a list of displayable artist names for this audio.
     *
     * @return list of displayable artist names.
     */
    function getArtistNames()
    {
        $ret = array();
        /* FIXME: maybe we can avoid copies in general by having a private
         * $this->artist_ids creator that we call */
        $artist_ids = $this->getArtistIds();
        /* FIXME: maybe make this an actual function to return the objects ? */
        /* if artist objects are not yet created, create them */
        if (!isset($this->artists)) {
            foreach ($artist_ids as $artist_id) {
                $this->artists[] = new DAD_artist($artist_id);
            }
        }
        foreach ($this->artists as $artist)
            $ret[] = $artist->getName();
        return $ret;
    }
    /**
     * Get a list of displayable artist links for this audio.
     *
     * @return list of displayable artist links.
     */
    function getArtistLinks()
    {
        $ret = array();
        /* FIXME: maybe we can avoid copies in general by having a private
         * $this->artist_ids creator that we call */
        $artist_ids = $this->getArtistIds();
        /* FIXME: maybe make this an actual function to return the objects ? */
        /* if artist objects are not yet created, create them */
        if (!isset($this->artists)) {
            foreach ($artist_ids as $artist_id) {
                $this->artists[] = new DAD_artist($artist_id);
            }
        }
        foreach ($this->artists as $artist)
            $ret[] = $artist->getLink();
        return $ret;
    }
    /**
     * Get a display for the audio that shows clickable links
     * for all artists, the audio, and the album if it's part of one.
     *
     * @return HTML snippet for the audio
     */
    /* FIXME: remove this, roll your own, it's easy now */
    function getFullDescription()
    {
        $artists = implode(" & ", $this->getArtistLinks());
        $audio = $this->getLink();
        $albumtext = "";
        if ($this->row->album_id > 0) {
            $album = new DAD_album($this->row->album_id);
            $albumtext = " (" . $album->getLink();
            $albumtext .= ", track " . $this->row->track . ")";
        }
        return "$artists - $audio$albumtext";
    }
    /**
     * Get an album object this audio is on.
     *
     * @return a DAD_album or none if this audio isn't on an album.
     */
    function getAlbum()
    {
        if ($this->row->album_id > 0) {
            return new DAD_album($this->row->album_id);
        } else {
            return null;
        }
    }
    /**
     * Get the ratings for all categories for a given user for this audio.
     *
     * @return array array of values of "un"/"na"/(0.0-1.0) keyed on cat_id.
     */
    function getUserRatings($user_id)
    {
        $result = array();
        $cat_ids =& $this->dad->getCatIds();
        /* by default they're na */
        foreach ($cat_ids as $cat_id) {
            $result[$cat_id] = "na";
        }
        /* get all unrated entries for this audio and user */
        $query = "SELECT cat_id FROM unrated "
               . "WHERE audio_id=" . $this->id . " AND user_id=$user_id";
        $this->dad->daddb->query($query);
        while ($row = $this->dad->daddb->row()) {
            $result[$row->cat_id] = "un";
        }
        /* get all actual scores for this audio and user */
        $query = "SELECT cat_id,rate FROM score "
               . "WHERE audio_id=" . $this->id . " AND user_id=$user_id";
        $this->dad->daddb->query($query);
        while ($row = $this->dad->daddb->row()) {
            $result[$row->cat_id] = $row->rate;
        }
        return $result;
    }

    /**
     * Set the rating for this audio for the given user_id and cat_id.
     *
     *
     * logic:
     * - if value = na, delete from unrated if it exists there, and
     *                  delete from scores if it exists there
     * - if value = un, insert in unrated if not exists there and
     *                  delete from scores if it exists there
     * - if value = numeric, delete from unrated if it exists there, and
     *                       update/insert in scores
     * return @boolean whether or not setting the rating succeeded.
     */
    function setRating($value, $user_id, $cat_id)
    {
        if (is_numeric ($value))
            if ($value < 0.0 || $value > 1.0)
                return FALSE;
        if (! is_numeric ($value))
            if ($value != "na" && $value != "un")
                return FALSE;
        $daddb =& $this->dad->daddb;
        $audio_id = $this->id;

        $daddb->begin ();
        
        /* first delete all values for this combination of audio/user/cat */
        $query = "DELETE FROM unrated "
               . "WHERE user_id='$user_id' AND audio_id='$audio_id' "
               . "AND cat_id='$cat_id'";
        $daddb->query($query);
      
        $query = "DELETE FROM score "
               . "WHERE user_id='$user_id' AND audio_id='$audio_id' "
               . "AND cat_id='$cat_id'";
        $daddb->query($query);
      
        /* now insert values as appropriate */
        if ($value == "un") {
            $query  = "INSERT INTO unrated "
                    . "SET audio_id = '$audio_id', cat_id = '$cat_id', "
                    . "user_id = '$user_id'";
            $daddb->query($query);
        } elseif (is_numeric ($value)) {
            $query = "INSERT INTO score "
                   . "SET audio_id = '$audio_id', cat_id = '$cat_id', "
                   . "user_id = '$user_id', rate = '$value'";
            $daddb->query($query);
        }
        $daddb->commit();
        return TRUE;
    }
}
?>
