<?php
/*
 * admin.inc.php
 * for now, this just does consistency checks on the database
 */

require_once 'DAD/audio.php';

/* check if all the audio files we have in the database exist on disk */
function check_audio_files ()
{
  global $daddb;
  global $AUDIO_DIR;

  $i = 0;
  $missing = array (); /* store the missing rows */

  $query  = "SELECT audiofile_id, filename, path ";
  $query .= "FROM audiofile, paths ";
  $query .= "WHERE audiofile.path_id = paths.path_id";

  if (!$daddb->query ($query))
    print "ERROR executing query $query<br>\n";

  print "<B>Checking " . $daddb->row_count () . " files: </B>\n";
  while ($row = $daddb->row ())
  {
    $i++;
    if ($i % 100 == 0)
    {
      print "$i, \n";
      flush ();
    }

    $filepath = $AUDIO_DIR . $row->path . "/" . $row->filename;
    if (!file_exists ($filepath))
      array_push ($missing, $row);
  }
  print "done.<BR>\n";

  foreach ($missing as $row)
  {
    $filepath = $AUDIO_DIR . $row->path . "/" . $row->filename;
    print "Missing " . $filepath . "<BR>\n";
  }
}

/* check if all the audio files mentioned in the .m3u files exist on disk */
function check_audio_files_in_m3us ()
{
  global $AUDIO_DIR;

  print "Looking for all .m3u files ...<BR>\n";
  flush ();
  $m3us = explode ("\n", rtrim (`find $AUDIO_DIR -name "*.m3u" 2> /dev/null`));
  array_pop ($m3us);

  $missing = array (); /* store the missing filenames */

  print "<B>Checking " . count ($m3us) . " .m3u files: </B>\n";
  foreach ($m3us as $m3u)
  {
    $i++;
    if ($i % 100 == 0)
    {
      print "$i, \n";
      flush ();
    }
    $files = file ($m3u);
    /* Check each of the files by using the .m3u file's path */
    $pathinfo = pathinfo ($m3u);
    $path = $pathinfo["dirname"];
    foreach ($files as $file)
    {
      $filepath = $path . "/" . rtrim ($file);
      if (!file_exists ($filepath))
        array_push ($missing, $filepath);
    }
  }
  print "done.<BR>\n";

  foreach ($missing as $filepath)
  {
    print "Missing " . $filepath . "<BR>\n";
  }
}

/* check for consistency of album titles
   - check capitalization
   - check if all the albums that exist have at least a song
     and an artist attached to it */
function check_album ()
{
  global $daddb;

  $i = 0;

  /* only check for wrong caps once we have better functions for checking */
  /*
  $wrongcaps = array ();

  $query  = "SELECT title ";
  $query .= "FROM album ";

  if (!$daddb->query ($query))
    print "ERROR executing query $query<br>\n";

  print "Checking capitalization<br>\n";
  while ($row = $daddb->row ())
  {
    $i++;
    if ($i % 100 == 0)
    {
      print "$i, \n";
      flush ();
    }

    $title = $row->title;
    if ($title != capitalize ($title))
      array_push ($wrongcaps, $row);
  }
  print "done.<BR>\n";
  */

  /* check for connection with artists  and songs */
  $query  = "SELECT title, album_id ";
  $query .= "FROM album ";

  $rows = $daddb->all_rows ($query);

  print "<B>Checking for albums with missing artist assocations ...</B><br>\n";
  foreach ($rows as $row)
  {
    $id = $row->album_id;

    $query  = "SELECT axu_id ";
    $query .= "FROM albumxartist ";
    $query .= "WHERE album_id=$id";
    if (!$daddb->query ($query))
      print "ERROR executing query $query<br>\n";
    if ($daddb->row_count () == 0)
    {
      print "album.album_id $id (" . $row->title . ") has no artist associated with it.<br>\n";
    }
  }
  print "done.<BR>\n";

  print "<B>Checking for albums with missing song assocations ...</B><br>\n";
  foreach ($rows as $row)
  {
    $id = $row->album_id;

    $query  = "SELECT audio_id ";
    $query .= "FROM audio ";
    $query .= "WHERE album_id=$id";
    if (!$daddb->query ($query))
      print "ERROR executing query $query<br>\n";
    if ($daddb->row_count () == 0)
    {
      print "album.album_id $id (" . $row->title . ") has no audio objects associated with it.<br>\n";
    }
  }
  print "done.<BR>\n";

  /*
  foreach ($wrongcaps as $row)
  {
    print "Wrong capitalization of " . $row->title . "<br>\n";
  }
  */
}

/* check for consistency of artist table
   - check capitalization
   - check if all the artists that exist have at least a song */
function check_artist ()
{
  global $daddb;

  $i = 0;

  /* only check for wrong caps once we have better functions for checking */
  /*
  $wrongcaps = array ();

  $query  = "SELECT title ";
  $query .= "FROM album ";

  if (!$daddb->query ($query))
    print "ERROR executing query $query<br>\n";

  print "Checking capitalization<br>\n";
  while ($row = $daddb->row ())
  {
    $i++;
    if ($i % 100 == 0)
    {
      print "$i, \n";
      flush ();
    }

    $title = $row->title;
    if ($title != capitalize ($title))
      array_push ($wrongcaps, $row);
  }
  print "done.<BR>\n";
  */

  /* check for connection with artists and songs */
  /* Only "Various Artists" is allowed to have no songs */
  $query  = "SELECT artistname, artist_id ";
  $query .= "FROM artist ";

  $rows = $daddb->all_rows ($query);

  print "<B>Checking for artists with missing song assocations ...</B><br>\n";
  foreach ($rows as $row)
  {
    $id = $row->artist_id;

    $query  = "SELECT axu_id ";
    $query .= "FROM audioxartist ";
    $query .= "WHERE artist_id=$id";
    if (!$daddb->query ($query))
      print "ERROR executing query $query<br>\n";
    if ($daddb->row_count () == 0 && $row->artistname != "Various Artists")
    {
      print "artist.artist_id $id (" . $row->artistname . ") has no audio objects associated with it.<br>\n";
    }
  }
  print "done.<BR>\n";


  /*
  foreach ($wrongcaps as $row)
  {
    print "Wrong capitalization of " . $row->title . "<br>\n";
  }
  */
}

/* check for consistency of audio table
   - check if each audio object has at least one artist
   - check if each audio object has an audio file
   - check if it has a track number if it also has an album_id
   - check if it has an album_id if it also has a track number
 */
function check_audio ()
{
  global $daddb;

  $i = 0;

  $query  = "SELECT audio_id, audiofile_id, album_id, track ";
  $query .= "FROM audio ";

  print "Getting all audio objects... <br>\n";
  $rows = $daddb->all_rows ($query);

  print "<B>Checking for audio objects with missing artist assocations ...</B><br>\n";
  foreach ($rows as $row)
  {
    $id = $row->audio_id;

    $query  = "SELECT axu_id ";
    $query .= "FROM audioxartist ";
    $query .= "WHERE audio_id=$id";
    if (!$daddb->query ($query))
      print "ERROR executing query $query<br>\n";
    if ($daddb->row_count () == 0)
    {
      print "audio.audio_id $id (" . $row->title . ") has no artists associated with it.<br>\n";
    }
  }

  print "<B>Checking for audio objects with missing audio file ...</B><br>\n";
  foreach ($rows as $row)
  {
    $id = $row->audio_id;
    $file_id = $row->audiofile_id;
    $audio = new DAD_audio ($id);

    $query  = "SELECT audiofile_id ";
    $query .= "FROM audiofile ";
    $query .= "WHERE audiofile_id=$file_id";
    if (!$daddb->query ($query))
      print "ERROR executing query $query<br>\n";
    if ($daddb->row_count () == 0)
    {
      print "audio.audio_id $id (" . $audio->row->title . ") has a missing audiofile_id $file_id associated with it.<br>\n";
    }
  }

  print "<B>Checking for audio objects with album_id but no track number ...</B><br>\n";
  foreach ($rows as $row)
  {
    $id = $row->audio_id;
    $album_id = $row->album_id;
    $track = $row->track;

    if ($album_id != 0 && $track == 0)
    {
      print "audio.audio_id $id (" . $audio->row->title . ") has an album id $album_id but missing track number.<br>\n";
    }
  }

  print "<B>Checking for audio objects with track number but no album id ...</B><br>\n";
  foreach ($rows as $row)
  {
    $id = $row->audio_id;
    $album_id = $row->album_id;
    $track = $row->track;

    if ($track != 0 && $album_id == 0)
    {
      print "audio.audio_id $id (" . $audio->row->title . ") has a track number $track but missing album id.<br>\n";
    }
  }

  /*
  foreach ($wrongcaps as $row)
  {
    print "Wrong capitalization of " . $row->title . "<br>\n";
  }
  */
}

/* main */
include ('header.inc.php');

echo "<table cellspacing=3 cellpadding=2>";
echo "<tr><td>";
echo "<h3>Administration</h3>";
echo "</td></tr>";
echo "<tr><td>";

$what = "";
if (!array_key_exists('check', $_GET)) {
  print '<TR><TD><A href="admin.php?check=all">Run all checks</A></TD></TR>';
  print '<TR><TD><A href="admin.php?check=audio_files">';
  print 'Check for missing audio files</A></TD></TR>';
  print '<TR><TD><A href="admin.php?check=audio_files_in_m3us">';
  print 'Check for missing audio files from .m3u files</A></TD></TR>';
  print '<TR><TD><A href="admin.php?check=album">';
  print 'Check for consistency of album table</A></TD></TR>';
  print '<TR><TD><A href="admin.php?check=artist">';
  print 'Check for consistency of artist table</A></TD></TR>';
  print '<TR><TD><A href="admin.php?check=audio">';
  print 'Check for consistency of audio table</A></TD></TR>';
} else {
  $what = $_GET['check'];
}
if ($what == "audio_files" || $what == "all")
  check_audio_files ();
if ($what == "audio_files_in_m3us" || $what == "all")
  check_audio_files_in_m3us ();
if ($what == "album" || $what == "all")
  check_album ();
if ($what == "artist" || $what == "all")
  check_artist ();
if ($what == "audio" || $what == "all")
  check_audio ();

echo "</td></tr>";
echo "</table>";
?>
