<?php
/* 
 * vi:si:et:sw=4:sts=4:ts=4
 *
 * DAD - Digital Audio Database
 *
 * browse_viewartist.inc.php: browse a given artist id (/album id)
 *
 *  Copyright (C) 2004 Thomas Vander Stichele
 *  Copyright (C) 2004 Kristof De Jaegher
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *
 */

require_once "DAD/album.php";
require_once "DAD/artist.php";
require_once "DAD/audio.php";

/* the artist id to browse is passed through GET */
/* optionally, an album_id could be given as well */

$panesize = 20; /* number of items per pane; FIXME: get from conf */

/* pull in the tab/pane javascript code */
include "lib/pane.js";

debug ("browse_viewartist.inc.php: start");

/* if we're browsing by artist, an artist_id is set in the GET request */
$artist_id = "";
$artist = null;
if (isset($_GET['artist_id'])) {
    $artist_id = $_GET['artist_id'];
    $artist = new DAD_artist($artist_id);
}
/* if we want to browse only an album, an album_id is set in the GET request */
$album_id = "";
if (isset($_GET['album_id']))
    $album_id = $_GET['album_id'];

print "<!-- artist_id: $artist_id - album_id: $album_id -->\n";
debug ("artist_id: $artist_id - album_id: $album_id");

print '<SCRIPT language="Javascript">' . "\n";
print "function selectalbum_id() {\n";
print "  index = document.albumform.album_id.selectedIndex;\n";
print '  document.location.href="browse.php?dad=viewartist';
print '&artist_id=' . $artist_id;
print '&album_id=" + document.albumform.album_id[index].value;' . "\n";
print "}\n";
print "</SCRIPT>\n";

global $dadid,$level;

$daddb->begin();

/* if an artist id was given, we can show a dropdown box for the artist's
 * albums */
$album_options = ''; /* each of the <OPTION>...</OPTION> lines for albums */
if ($artist_id) {
    $album_ids = $artist->getAlbumIds();
    if ($album_ids) {
        $album_options .= "<OPTION value=0>No album, just all songs</OPTION>\n";
        foreach ($album_ids as $id) { 
            $album = new DAD_album($id);
            $album_options .= "<OPTION value=$id";
            if ($id == $album_id)
                $album_options .= " selected";
            $album_options .= ">" . $album->getTitle() . "</OPTION>\n";
        }
    }
}
?>
<TABLE border=0 cellspacing=0 cellpadding=0 class="menu" width="100%">
  <TR>
    <TD height=20 class="menu">
      <IMG src="images/spacer.gif" width=1 height=20>
    </TD>
    <TD nowrap class="menu">
      <B>
<?php

/* 
 * the block under the header:
 * - shows the artist
 * - has a link to your ratings
 * - lists all the albums by this artist in a drop-down box
 */

if ($artist) {
    print "All songs by " . $artist->getLink();
    if ($dadid) {
        print " - ";
        print '<A class=menu href="browse.php?dad=ratings&artist_id=';
        print $artist_id . '">';
        print "Your Ratings</A>\n";
    }
}
?>
      </B>     
    </TD>
<?php
/* if there are albums for this artist, show the drop down box */
if ($album_options) {
    print "<TD class=menu nowrap>&nbsp;- <B>Albums : </B></TD>\n";
    print "<TD width=100% class=\"menu\">\n";
    print "  <FORM action=browse.php method=POST name=albumform>\n";
    print "    <INPUT type=hidden name=dad value=viewartist></INPUT>\n";
    print "    <INPUT type=hidden name=artist_id value=$artist_id></INPUT>\n";
    print '    <INPUT type=hidden name=artistname value="';
    print $artist->getName();
    print '">' . "\n";
    print "    <SELECT name=album_id onchange=\"selectalbum_id()\">\n";
    print "$album_options";
    print "    </SELECT>\n";
    print "  </FORM>\n";
    print "</TD>\n";
}
?>
  </TR>
  <TR>
    <TD class="divide" colspan=5>
      <IMG src="images/spacer.gif" width=1 height=1>
    </TD>
  </TR>
</TABLE>
<?php
/* if no album id was given through GET, we list all the tracks this artist
 * has linked */
if (!$album_id)
{
    $audio_ids = $artist->getAudioIds();
    $items = count($audio_ids);

    /* construct audio tabs */
    echo "\n<!-- audio tabs -->\n";
    echo "<TABLE border=0 cellspacing=2 cellpadding=2>\n";
    echo "<TR>\n";
     
    for ($i = 0; $i < $items; $i += $panesize) {
        $tab = $i / $panesize + 1; /* no intval needed here */
        $lower = $i + 1;
        $upper = $i + $panesize;
        if ($upper > $items) $upper = $items;
     
        echo '<TD><DIV id="tab_' . $tab .'" class="tab" ';
        echo 'onClick="showPane(' . $tab . ');" ';
        echo 'onMouseOver="hover(this);" ';
        echo 'onMouseOut="setState(' . $tab . ')">';
        echo "Songs $lower - $upper</DIV></TD>\n";
    }
    echo "</TR>\n";
    echo "</TABLE>\n";

    /* construct artist panes */
    echo "\n<!-- artist panes -->\n";
    echo '<TABLE border=0 cellspacing=2 cellpadding=2 width="90%">';
    echo "\n";
    echo "<TR>\n<TD>\n";
    $item = 1; /* going from 1 to items */
    foreach ($audio_ids as $audio_id) {
        $pane = intval(($item - 1) / $panesize) + 1;
        /* if item is a multiple of panesize + 1, it's the first on a pane */
        if ($item % $panesize == 1) {
            $pane = intval ($item / $panesize) + 1;
            echo '<DIV id="pane_' . $pane . '" class="pane">';
            echo "\n";
            echo "<TABLE cellspacing=1 cellpadding=1 border=0>\n";
        }
        /* print the song info */
        $audio = new DAD_audio($audio_id);
        print "<TR>";
        /* only show playlist add/play links if logged in */
        if ($dadid) {
            print "<TD>";
            /* FIXME: make AddPlayListLinks return <TD></TD> bits */
            print $audio->getAddPlaylistLinks($dadid, $pane);
            print "</TD><TD>\n";
            print $audio->getPlayLink($dadid, 1, "Stream audio") . "\n";
            print $audio->getPlayLink($dadid, 2, "Play audio locally") . "\n";
            print "<TD>";
        }
        print "<TD>$item) ";
        /* if logged in, show clickable title, else just title */
        if ($dadid)
          print $audio->getLink();
        else
          print $audio->getTitle();
        print "</TD>";
        print "<TD>&nbsp;";
        /* if on an album, display that as well */
        if ($audio->row->album_id) {
            print " (";
            $album = new DAD_album($audio->row->album_id);
            if ($dadid)
                print $album->getLink();
            else
                print $album->getTitle();
            print ")";
        }
        print "</TD></TR>\n";

      /* if item is a multiple of panesize, or equal to rows, it's the last */
      if (($item % $panesize == 0) or ($item == $items)) {
        echo "</TABLE>\n";
        echo "</DIV>\n\n";
      }
      ++$item;
    }
    echo "</TD>\n</TR>\n</TABLE>\n";
} else {
  /* an album id was given, show this instead */
  /* FIXME: this really should just be a function that takes content rows
   * and displays tabs and panes */
    $album = new DAD_album($album_id);
    print '<DIV id="pane_1" class="pane">' . "\n";
    print "<TABLE cellspacing=1 cellpadding=0 border=0>\n<TR>\n<TD></TD>\n";
    print "<TD colspan=2>\n";
    if ($dadid) {
        print $album->getPlayLink($dadid, 1, "Stream album") . "\n";
        print $album->getPlayLink($dadid, 2, "Play album locally") . "\n";
        print "&nbsp;&nbsp;";
    }
    print "</TD>\n<TD><B class=menu>Album: ";
    print $album->getTitle();
    print "</B></TD>\n</TR>\n";

    /* get all the tracks from this album */
    $query  = "SELECT audio_id FROM audio ";
    $query .= "WHERE album_id='$album_id' ORDER BY track ASC";
    $rows = $daddb->all_rows($query);
    for ($i = 0; $i < count($rows); $i++) {
        $audio = new DAD_audio($rows[$i]->audio_id);
        print "<TR>";
        if ($dadid) {
            print "<TD>";
            print $audio->getAddPlaylistLinks($dadid, 1);
            print "</TD><TD>\n";
            print $audio->getPlayLink($dadid, 1, "Stream audio") . "\n";
            print $audio->getPlayLink($dadid, 2, "Play audio locally") . "\n";
            print "<TD>";
        }
        print "<TD>" . ($i + 1) . ") ";
        if ($dadid)
            print $audio->getLink();
        else
            print $audio->getTitle();
        print "</TD></TR>\n";
    }
    print "</TABLE>\n";
    print "</DIV>\n";           
}
$daddb->commit();
?>
