<?php

require_once "DAD/album.php";
require_once "DAD/artist.php";
require_once "DAD/audio.php";

/**
 * functions useful in rating songs
 */

/* helper array_walk function to prepend an array of lines with a prefix */

function prepend (&$line, $key, $prefix) { $line = "$prefix$line"; }

/* construct an HTML table for rating the given array of audio id's
 * if only_unrated is TRUE, show only unrated categories
 * returns an array of lines to be exploded with spaces */
function audio_ids_rate_table ($audio_ids, $refresh, $only_unrated = TRUE)
{
  global $dadid;

  debug ("audio_ids_rate_table: start");

  $res = array ();

  $audio_id_list = implode (",", $audio_ids);
  $res[] = "<FORM action=browse.php?dad=ratings_store method=post>";
  $res[] = "  <INPUT type=hidden name=user_id value=$dadid>";
  $res[] = "  <INPUT type=hidden name=audio_ids value=$audio_id_list>";
  $res[] = "  <INPUT type=hidden name=refresh value=\"$refresh\">";
  $res[] = "  <TABLE cellspacing=2 cellpadding=2 border=0>";
  $res[] = "  <TR>";
  $res[] = "    <TD>";
  $res[] = "      <TABLE width=100% cellspacing=3 cellpadding=3 border=0 class=\"searchtable\">";
  $res[] = "      <TR>";
  $res[] = "        <TD colspan=2><B><FONT size=+1>Ratings</FONT></B></TD>";
  $res[] = "      </TR>";

  /* pull in audio rate blocks */ 
  foreach ($audio_ids as $audio_id)
  {
    debug ("audio_ids_rate_table: getting audio_rate_block for $audio_id");
    $block = audio_rate_block ($audio_id, $only_unrated);
    array_walk ($block, 'prepend', '        ');
    foreach ($block as $line) $res[] = $line;
  }

  $res[] = "      <TR><TD><INPUT type=submit value=\"Rate\"></TD></TR>";
  $res[] = "      </TABLE>";
  $res[] = "    </TD>";
  $res[] = "  </TR>";
  $res[] = "  </TABLE>";
  $res[] = "</FORM>";

  debug ("audio_ids_rate_table: returning");
  return $res;
}

/* construct an HTML block for playback of this track */
function audio_play_block ($audio_id)
{
  global $dadid; /* current user id */
  global $daddb;

  $res = array ();
  $daddb->begin ();
  $query = "SELECT audiofile_id FROM audio WHERE audio_id = $audio_id";
  if (!$daddb->query ($query))
    return ("(ERROR: query $query failed)");
  if ($daddb->row_count () == 0)
    return ("(ERROR: id $audio_id not found)");
  $row = $daddb->row ();
  $file_id = $row->audiofile_id;

  $alt = "Stream song over network";
  $res[] = "<A href=playlist.m3u?dad=song&audiofileid=$file_id&dadid=$dadid&type=1>";
  $res[] = "<IMG src=images/sound1.gif border=0 alt=\"$alt\" title=\"$alt\"></A>";
  $res[] = "&nbsp;";
  $alt = "Play file locally";
  $res[] = "<A href=playlist.m3u?dad=song&audiofileid=$file_id&dadid=$dadid&type=2>";
  $res[] = "<IMG src=images/sound2.gif border=0 alt=\"$alt\" title=\"$alt\"></A>";

  return $res;
}

/* construct an HTML block to rate the given audio (by id)
 * if only_unrated is TRUE, show only unrated categories
 * block will be returned as an array of lines, being table rows */
function audio_rate_block ($audio_id, $only_unrated = TRUE)
{
  /* FIXME: unglobalize $dadid somehow */
  global $dadid; /* current user id */
  debug ("audio_rate_block: start for $audio_id");
  $res = array ();

  $audio = new DAD_audio($audio_id);
  $ratings = $audio->getUserRatings($dadid);

  /* if only_unrated is TRUE, and all categories are rated, then just
   * return the empty array */
  if (! in_array("un", $ratings)) {
    $res[] = "";
    return $res;
  }

  $album = $audio->getAlbum();
  $title = $audio->getFullDescription();

  $res[] = "<TR>";
  $res[] = "  <TD align=middle>";

  /* pull in play block */
  $block = audio_play_block ($audio_id);
  array_walk ($block, 'prepend', '    ');
  foreach ($block as $line) $res[] = $line;
  $res[] = "  <TD><B>" . implode(" & ", $audio->getArtistLinks())
         . " - " . $audio->getLink() . "</B></TD>"; 
  /* add an album link and track number if there is one */
  if ($album) {
    $res[] = '  <TD align="right"><B>' . $album->getLink()
           . " (" . $audio->row->track . ")</B></TD>";
  }
  $res[] = "</TR>";

  $cats = dad_get_table ("category", "cat_id");
  foreach ($ratings as $cat_id => $rating)
  {
    if ($only_unrated && $rating != "un") continue;

    $res[] = "<TR>";
    $res[] = "  <!-- rating: $rating -->";
    $res[] = "  <TD><B>" . $cats[$cat_id]->description . "</B></TD>";
    $res[] = "  <TD colspan=2>";

    # radio button for unrated
    $line =  "    <INPUT type=radio value=un   name=catsel[$audio_id][$cat_id]";
    if ($rating == "un") $line .= " checked";
    $line .= "> Unrated";
    $res[] = $line;
    
    # radio button for 0
    # put "zero" for a score of 0 so it still gets passed in an array
    $line =  "    <INPUT type=radio value=zero name=catsel[$audio_id][$cat_id]";
    if (is_numeric ($rating) && $rating == 0) $line .= " checked";
    $line .= "> 0";
    $res[] = $line;

    # radio buttons for 100 to 1000
    for ($i = 1; $i < 11; ++$i)
    {
      $value = $i * 100;
      $line  = "    <INPUT type=radio value=$value  ";
      $line .= "name=catsel[$audio_id][$cat_id]";
      if (is_numeric ($rating))
        if ($value == $rating * 1000.0)
          $line .= " checked";
      $line .= "> $value";
      $res[] = $line;
    }
    $line = "     <INPUT type=radio value=na name=catsel[$audio_id][$cat_id]";
    if ($rating == "na") $line .= " checked";
    $line .= "> Not applicable";

    # input field
    $line  = "    <INPUT type=text  size=5     ";
    $line .= "name=catent[$audio_id][$cat_id]";
    if (intval ($rating * 1000.0) % 100 != 0)
      $line .= " value=" . (intval ($rating * 1000.0));
    $line .= ">";
    $res[] = $line;

    # finish up
    $res[] = "  </TD>";
    $res[] = "</TR>";
  }

  debug ("audio_rate_block: returning result");
  return $res;
}
?>
